static PyObject* event_getattr(GIIEventObject *self, char *name) {
	PyObject *a;
	unsigned int i;
	switch(name[0]) {
	case 'e':
		if (!strcmp(name,"error"))
			return PyInt_FromLong(self->ev.any.error);
		break;
	case 'o':
		if (!strcmp(name,"origin"))
			return PyInt_FromLong(self->ev.any.origin);
		break;
	case 't':
		if (!strcmp(name,"type"))
			return PyInt_FromLong(self->ev.any.type);
		if (!strcmp(name,"target"))
			return PyInt_FromLong(self->ev.any.target);
		if (!strcmp(name,"time"))
			return Py_BuildValue("[ll]",
					     self->ev.any.time.tv_sec,
					     self->ev.any.time.tv_usec);
		break;
	}
	
	switch (self->ev.any.type) {
	case evKeyPress:
	case evKeyRelease:
	case evKeyRepeat:
		if(!strcmp(name,"modifiers"))
			return PyInt_FromLong(self->ev.key.modifiers);
		if (!strcmp(name,"sym"))
			return PyInt_FromLong(self->ev.key.sym);
		if (!strcmp(name,"label"))
			return PyInt_FromLong(self->ev.key.label);
		if (!strcmp(name,"button"))
			return PyInt_FromLong(self->ev.key.button);
		break;
		
	case evPtrRelative:
	case evPtrAbsolute:
		if (!strcmp(name,"x"))
			return PyInt_FromLong(self->ev.pmove.x);
		if (!strcmp(name,"y"))
			return PyInt_FromLong(self->ev.pmove.y);
		if (!strcmp(name,"z"))
			return PyInt_FromLong(self->ev.pmove.z);
		if (!strcmp(name,"wheel"))
			return PyInt_FromLong(self->ev.pmove.wheel);
		break;
		
	case evPtrButtonPress:
	case evPtrButtonRelease:
		if (!strcmp(name,"button"))
			return PyInt_FromLong(self->ev.pbutton.button);
		break;
		
	case evExpose:
		if (!strcmp(name,"x"))
			return PyInt_FromLong(self->ev.expose.x);
		if (!strcmp(name,"y"))
			return PyInt_FromLong(self->ev.expose.y);
		if (!strcmp(name,"h"))
			return PyInt_FromLong(self->ev.expose.h);
		if (!strcmp(name,"w"))
			return PyInt_FromLong(self->ev.expose.w);
		break;
		
	case evCommand:
	case evInformation:
		if (!strcmp(name, "code"))
			return PyInt_FromLong(self->ev.cmd.code);
		if (!(self->ev.cmd.code & GII_CMDFLAG_NODATA))
			if (!strcmp(name, "data"))
				return PyString_FromStringAndSize(
					self->ev.cmd.data,
					self->ev.cmd.size-sizeof(gii_cmd_nodata_event));
		break;
		
	case evValRelative:
	case evValAbsolute:
		if (!strcmp(name,"first"))
			return PyInt_FromLong(self->ev.val.first);
		if (!strcmp(name,"count"))
			return PyInt_FromLong(self->ev.val.count);
		if (!strcmp(name,"value")) {
			a=PyList_New(self->ev.val.count);
			if(a)
				for(i=0;i<self->ev.val.count;i++)
					PyList_SET_ITEM(a,i,
							PyInt_FromLong(self->ev.val.value[i]));
			return a;
		}
		break;
	default:
	}
	PyErr_SetString(PyExc_AttributeError, name);
	return NULL;
}


static int event_setattr(GIIEventObject *self, char *name, PyObject *value) {
	unsigned int i;
	int n;
	int v[32];
	char * c;
	switch(name[0]) {
	case 'e':
		if (!strcmp(name,"error")) {
			if(pygii_get_int(value,&n)<0)
				return -1;
			self->ev.any.error=n;
			return 0;
		}
		break;
	case 'o':
		if (!strcmp(name,"origin"))
			return pygii_get_int(value,&self->ev.any.origin);
		break;
	case 't':
		if (!strcmp(name,"type")) {
			if(pygii_get_int(value,&i)<0)
				return -1;
			switch(i) {
			case evCommand:
			case evInformation:
				/* this will actualy depend on the code itself. */
				self->ev.any.size=sizeof(gii_cmd_event);
				break;
			case evExpose:
				self->ev.any.size=sizeof(gii_expose_event);
				break;
			case evKeyPress:
			case evKeyRelease:
			case evKeyRepeat:
				self->ev.any.size=sizeof(gii_key_event);
				break;
			case evPtrButtonPress:
			case evPtrButtonRelease:
				self->ev.any.size=sizeof(gii_pbutton_event);
				break;
			case evPtrRelative:
			case evPtrAbsolute:
				self->ev.any.size=sizeof(gii_pmove_event);
				break;
			case evValRelative:
			case evValAbsolute:
				self->ev.any.size=sizeof(gii_val_event);
				break;
			default:
				PyErr_SetString(PyExc_ValueError, name);
				return -1;
			}
			self->ev.any.type=i;
			return 0;
		}
		if (!strcmp(name,"target"))
			return pygii_get_int(value,&self->ev.any.target);
		if (!strcmp(name,"time"))
			return pygii_get_time(value,&self->ev.any.time);
		break;
	}
	
	switch (self->ev.any.type) {
	case evKeyPress:
	case evKeyRelease:
	case evKeyRepeat:
		if(!strcmp(name,"modifiers"))
			return pygii_get_int(value,&self->ev.key.modifiers);
		if (!strcmp(name,"sym"))
			return pygii_get_int(value,&self->ev.key.sym);
		if (!strcmp(name,"label"))
			return pygii_get_int(value,&self->ev.key.label);
		if (!strcmp(name,"button"))
			return pygii_get_int(value,&self->ev.key.button);
		break;
		
	case evPtrRelative:
	case evPtrAbsolute:
		if (!strcmp(name,"x"))
			return pygii_get_int(value,&self->ev.pmove.x);
		if (!strcmp(name,"y"))
			return pygii_get_int(value,&self->ev.pmove.y);
		if (!strcmp(name,"z"))
			return pygii_get_int(value,&self->ev.pmove.z);
		if (!strcmp(name,"wheel"))
			return pygii_get_int(value,&self->ev.pmove.wheel);
		break;
		
	case evPtrButtonPress:
	case evPtrButtonRelease:
		if (!strcmp(name,"button"))
			return pygii_get_int(value,&self->ev.pbutton.button);
		break;
		
	case evExpose:
		if (!strcmp(name,"x"))
			return pygii_get_int(value,&self->ev.expose.x);
		if (!strcmp(name,"y"))
			return pygii_get_int(value,&self->ev.expose.y);
		if (!strcmp(name,"h"))
			return pygii_get_int(value,&self->ev.expose.h);
		if (!strcmp(name,"w"))
			return pygii_get_int(value,&self->ev.expose.w);
		break;
		
	case evCommand:
	case evInformation:
		if (!strcmp(name, "code"))
			return pygii_get_int(value,&self->ev.cmd.code);
		if (!(self->ev.cmd.code & GII_CMDFLAG_NODATA))
			if (!strcmp(name, "data")) {
				n=PyString_Size(value);
				if (PyErr_Occurred())
					return -1;
				if(n>GII_CMD_DATA_MAX) {
					PyErr_SetString(PyExc_ValueError, name);
					return -1;
				}
				c=PyString_AsString(value);
				memmove(self->ev.cmd.data,c,n);
				self->ev.any.size=sizeof(gii_cmd_event)-
					(GII_CMD_DATA_MAX-n);
				return 0;
			}
		break;
		
	case evValRelative:
	case evValAbsolute:
		if (!strcmp(name,"first"))
			return pygii_get_int(value,&self->ev.val.first);

		if (!strcmp(name,"value")) {
			n=PyList_Size(value);
			if (PyErr_Occurred()) return -1;
			if(n>32) {
				PyErr_SetString(PyExc_ValueError, name);
				return -1;
			}
			for(i=0;i<n;i++)
				if(pygii_get_int(value,&v[i])<0)
					return -1;
			for(i=0;i<n;i++)
				self->ev.val.value[i]=v[i];
			self->ev.val.count=n;
			return 0;
		}
		break;
	default:
	}
	PyErr_SetString(PyExc_AttributeError, name);
	return -1;
}

static void event_dealloc(GIIEventObject *self) {
	PyObject_Del(self);
}

#ifndef PRE_2_2


static char * event_names[]={
	"Nothing",
	"Command",
	"Information",
	"Expose",
	"",
	"KeyPress",
	"KeyRelease",
	"KeyRepeat",
	"PtrRelative",
	"PtrAbsolute",
	"PtrButtonPress",
	"PtrButtonRelease",
	"ValRelative",
	"ValAbsolute"
};

static PyObject * event_str(GIIEventObject * self) {
	PyObject * common;
	PyObject * add;

	if(!(self->ev.any.type>0 &&
	     self->ev.any.type<evLast &&
	     self->ev.any.type!=evExpose+1)) {
		return PyString_FromString("Invalid event");
	}

	common=PyString_FromFormat(
		"size: %2x error: %4x origin: %8x target: %8x time: %is,%ius type: %s",
		self->ev.any.size,
		self->ev.any.error,
		self->ev.any.origin,
		self->ev.any.target,
		self->ev.any.time.tv_sec, self->ev.any.time.tv_usec,
		event_names[self->ev.any.type]);
	add=NULL;
	switch(self->ev.any.type) {
	case evKeyPress:
	case evKeyRepeat:
	case evKeyRelease:
		add=PyString_FromFormat(
			" [ modifier:%4x sym:%4x label:%4x button:%4x ]",
			self->ev.key.modifiers,
			self->ev.key.sym,
			self->ev.key.label,
			self->ev.key.button);
		break;
	case evValAbsolute:
	case evValRelative:
		add=PyString_FromFormat(
			" [ first:%8x count:%8x ]",
			self->ev.val.first,self->ev.val.count);
		break;
	case evPtrAbsolute:
	case evPtrRelative:
		add=PyString_FromFormat(
			" [ x:%4x y:%4x z:%4x wheel:%4x ]",
			self->ev.pmove.x,self->ev.pmove.y,self->ev.pmove.z,
			self->ev.pmove.wheel);
		break;
	case evPtrButtonPress:
	case evPtrButtonRelease:
		add=PyString_FromFormat(" [ button:%4x ]",self->ev.pbutton.button);
		break;
	case evExpose:
		add=PyString_FromFormat(
			" [ x:%4x y:%4x w:%4x h:%4x ]",
			self->ev.expose.x,self->ev.expose.y,
			self->ev.expose.w,self->ev.expose.h);
		break;
	case evCommand:
	case evInformation:
		add=PyString_FromFormat(
			" [ code:%4x ]",
			self->ev.cmd.code);
		break;
	default:
	}
	if(add)
		PyString_ConcatAndDel(&common,add);
	return common;
}
#endif /* PRE_2_2 */

const char event_doc[]="
An event object in the GII event model.

Common fields:

- type   : one of evNothing(invalid event), evCommand,evInformation,
- error  : the error code set by a device when replying to a command
- target : the target of this event. It is used for user event sent to the
           queue. It can be one of evTargetQueue (this event is queued
           normally), evTargetAll (the event is sent to each device) or an
           origin (the event is sent only to the designated device).
- origin : the device that generated this event, in case of a normal
           event. In case of a user event, it can be evOriginNone
           (anonymous event) or evOriginSendEvent (normal user event
           injected in the queue).
- time   : the time when this event was generated. It is a list giving
           the number of second and microsecond in system time (as in
           the timeval struct).
- size   : the size of this event. This value is read-only. It is auto-
           matically updated when the event is altered.

Depending on the event type, other fields can be accessed:

- evCommand , evInformation: 
           - code : the command code. useful ones are those with
                    the cmd- prefix and flagged by the cmdflag-
                    constants in the pygii namespace.
           - data : A binary string associated with the command. It is
                    only available if the code doesn't have the cmdflagNoData
                    bit set. Setting the data will update the size of the
                    event.

- evPtrButtonPress, evPtrButtonRelease:
           - button : the number of the button which changed state.
                      For mouse buttons, the constant pbuttonLeft,
                      pbuttonMiddle and pbuttonRight are defined.

- evPtrAbsolute, evPtrRelative:
           - x,y,z and wheel  : the pointer motion, either absolute or relative,
                                in the 4 possible dimensions of the device.

- evKeyPress, evKeyRepeat, evKeyRelease:
           - modifiers : the modifiers that were active when the event occured.
           - sym       : the resultant character produced by the key
           - label     : can be used as a generalized, portable keycode or
                         scancode of the key.
           - button    : the scancode of the key.

- evValAbsolute, evValRelative:
           - first : The index of the first valuator for which values are reported
           - count : A read-only field that gives the number of values reported
                     in this event. It cannot exceed 32.
           - value : An array holding the reported values. The length is equal to
                     the count value. The value can be set by giving a list of
                     integer with a maximum of 32 values. The values are affected
                     and the count field is updated.

- evExpose :
           - x,y,w,h : The bounding box of the area that needs to be repainted.

";

static PyTypeObject event_type = {
	PyObject_HEAD_INIT(NULL)
	0,
	"GII Event",
	sizeof(GIIEventObject),
	0,
	
	/* Methods to implement standard operations */
	
	(destructor)event_dealloc,
	(printfunc)0,
	(getattrfunc)event_getattr,
	(setattrfunc)event_setattr,
	(cmpfunc)0,
	(reprfunc)0,

        /* Method suites for standard classes */

	0,
	0,
	0,

	/* More standard operations (here for binary compatibility) */

	(hashfunc)0,
	(ternaryfunc)0,
#ifndef PRE_2_2
	(reprfunc)event_str,
#else
	(reprfunc)0,
#endif
	0L,0L,0L,0L,
	(char*)event_doc
};

static PyObject* newEventObjectFromEvent(gii_event e) {
	GIIEventObject *self;
	self=PyObject_New(GIIEventObject, &event_type);
	if(!self) return NULL;
	self->ev=e;
	return(PyObject*)self;
};

const char event_construct_doc[]="
Construct a new Event object. Type is set to evNothing (and must be changed
before use), error to 0, origin to GII_EV_ORIGIN_SENDEVENT (as it is the most
probable use), target to GII_EV_TARGET_QUEUE, and the time is set to the
current time.

Parameter

     - None

Return Value

     - a new event.

Exception

     - None

----------------------------------
A word on python GII Event objects
----------------------------------

Common fields:

- type   : one of evNothing(invalid event), evCommand,evInformation,
- error  : the error code set by a device when replying to a command
- target : the target of this event. It is used for user event sent to the
           queue. It can be one of evTargetQueue (this event is queued
           normally), evTargetAll (the event is sent to each device) or an
           origin (the event is sent only to the designated device).
- origin : the device that generated this event, in case of a normal
           event. In case of a user event, it can be evOriginNone
           (anonymous event) or evOriginSendEvent (normal user event
           injected in the queue).
- time   : the time when this event was generated. It is a list giving
           the number of second and microsecond in system time (as in
           the timeval struct).
- size   : the size of this event. This value is read-only. It is auto-
           matically updated when the event is altered.

Depending on the event type, other fields can be accessed:

- evCommand , evInformation: 
           - code : the command code. useful ones are those with
                    the cmd- prefix and flagged by the cmdflag-
                    constants in the pygii namespace.
           - data : A binary string associated with the command. It is
                    only available if the code doesn't have the cmdflagNoData
                    bit set. Setting the data will update the size of the
                    event.

- evPtrButtonPress, evPtrButtonRelease:
           - button : the number of the button which changed state.
                      For mouse buttons, the constant pbuttonLeft,
                      pbuttonMiddle and pbuttonRight are defined.

- evPtrAbsolute, evPtrRelative:
           - x,y,z and wheel  : the pointer motion, either absolute or
                                relative, in the 4 possible dimensions
                                of the device.

- evKeyPress, evKeyRepeat, evKeyRelease:
           - modifiers : the modifiers that were active when the event occured.
           - sym       : the resultant character produced by the key.
           - label     : can be used as a generalized, portable keycode or
                         scancode of the key.
           - button    : the scancode of the key.

- evValAbsolute, evValRelative:
           - first : The index of the first valuator for which values are
                     reported.
           - count : A read-only field that gives the number of values reported
                     in this event. It cannot exceed 32.
           - value : An array holding the reported values. The length is equal
                     to the count value. The value can be set by giving a list
                     of integers with a maximum of 32 values. The values are
                     affected and the count field is updated.

- evExpose :
           - x,y,w,h : The bounding box of the area that needs to be repainted.
";

static PyObject* EventConstruct(void) {
	GIIEventObject *self;
	self=PyObject_New(GIIEventObject, &event_type);
	if(!self) return NULL;
	self->ev.any.size=0;
	self->ev.any.type=evNothing;
	self->ev.any.origin=GII_EV_ORIGIN_SENDEVENT;
	self->ev.any.error=0;
	self->ev.any.target=GII_EV_TARGET_QUEUE;
	ggCurTime(&self->ev.any.time);
	return(PyObject*)self;
}
